/*
	CLONE_8722
	
	device: XC95108-7 PC84 (PLCC 84) 
	
	An attempt to clone some functions found in mmu CBM8722
	
	This implementation clone these registers (all read/write):
	
	MCR  - memory configuration register (as reg. $FF00/$D500 in true 8722)
	PCRx - 4 preconfiguration register (as reg's $D501-$D504 in true 8722)
	CR   - control register for mapping common ram
	PR   - page 0 register relocation
	
	Bit 7&6 of MCR select one of 4 64K ram bank - others bits are free to
	implement user function
	bit 0&1 of CR select common ram (if any)> 1K, 2K, 4K, 8K
	common ram lies always in ram bank 0
	bit 2 of CR enable common ram (otherwise just page 0 and page 1 lies in
	ram bank 0)
	others bit of CR are free to implement user functions

	PR   - set relocation of page 0 (in fact swap 2 page of ram bank 0)
	page 0 & 1 lies always in ram bank 0
	
	Preconfiguration mechanism work as in true 8722: storing a byte in 
	PCR0-PCR3 don't change MC7..MC0 bit< writing to LCR0-LCR3 transfer
	the content of PCR0-PCR3 into MCR
	
	Storing a byte into MCR affect directly MC7..MC0 bits

	NOTE: cannot be used XC9572 because this implementation need 78 macrocells
*/

`timescale 1ns / 1ps

// MUX 2 TO 1
module mx2e1(A, B, E, S0, Y);
	input A;
	input B;
	input E;
	input S0;
	output Y;
   
   wire YA = A & E & !S0;
   wire YB = B & E &  S0;

	assign Y = YA | YB;

endmodule

// BUS 8 BITS MUX 2 TO 1 WITH ENABLE
module mx2e(X0, X1, E, S0, Y);

	input [7:0] X0;
	input [7:0] X1;
	input E;
	input S0;
	(* NOREDUCE = "YES,YES,YES,YES,YES,YES,YES,YES" *) 
	output [7:0] Y;

	wire Y0;
	wire Y1;
	wire Y2;
	wire Y3;
	wire Y4;
	wire Y5;
	wire Y6;
	wire Y7;

	mx2e1 MX21E_0 (.A(X0[0]), .B(X1[0]), .E(E), .S0(S0), .Y(Y0));
	mx2e1 MX21E_1 (.A(X0[1]), .B(X1[1]), .E(E), .S0(S0), .Y(Y1));
	mx2e1 MX21E_2 (.A(X0[2]), .B(X1[2]), .E(E), .S0(S0), .Y(Y2));
	mx2e1 MX21E_3 (.A(X0[3]), .B(X1[3]), .E(E), .S0(S0), .Y(Y3));
	mx2e1 MX21E_4 (.A(X0[4]), .B(X1[4]), .E(E), .S0(S0), .Y(Y4));
	mx2e1 MX21E_5 (.A(X0[5]), .B(X1[5]), .E(E), .S0(S0), .Y(Y5));
	mx2e1 MX21E_6 (.A(X0[6]), .B(X1[6]), .E(E), .S0(S0), .Y(Y6));
	mx2e1 MX21E_7 (.A(X0[7]), .B(X1[7]), .E(E), .S0(S0), .Y(Y7));

	assign Y[0] = Y0;
	assign Y[1] = Y1;
	assign Y[2] = Y2;
	assign Y[3] = Y3;
	assign Y[4] = Y4;
	assign Y[5] = Y5;
	assign Y[6] = Y6;
	assign Y[7] = Y7;

endmodule

// MUX 5 TO 1
module mx51(
	  input A,
	  input B,
	  input C,
	  input D,
	  input E,
	  input S0,
	  input S1,
	  input S2,	  
	  output Y);

	wire SLA = (!S0 & !S1 & !S2);
	wire SLB = ( S0 & !S1 & !S2);
	wire SLC = (!S0 &  S1 & !S2);
	wire SLD = ( S0 &  S1 & !S2);
	wire SLE = S2;
	
	wire YA = A & SLA;
	wire YB = B & SLB;
	wire YC = C & SLC;
	wire YD = D & SLD;
	wire YE = E & SLE;
	
	assign Y = (YA | YB | YC | YD | YE);
endmodule

// BUS 8 BITS MUX 5 TO 1
module mx5(
    input [7:0] X0,
    input [7:0] X1,
    input [7:0] X2,
    input [7:0] X3,
    input [7:0] XX,
    input A0,
    input A1,
    input AX,
    output [7:0] Y
    );

	wire Y0;
	wire Y1;
	wire Y2;
	wire Y3;
	wire Y4;
	wire Y5;
	wire Y6;
	wire Y7;
	
	mx51 MX51_0 (.A(X0[0]), .B(X1[0]), .C(X2[0]), 
						 .D(X3[0]), .E(XX[0]), .S0(A0), 
						 .S1(A1), .S2(AX), .Y(Y0));

	mx51 MX51_1 (.A(X0[1]), .B(X1[1]), .C(X2[1]), 
						 .D(X3[1]), .E(XX[1]), .S0(A0), 
						 .S1(A1), .S2(AX), .Y(Y1));
						 
	mx51 MX51_2 (.A(X0[2]), .B(X1[2]), .C(X2[2]), 
						 .D(X3[2]), .E(XX[2]), .S0(A0), 
						 .S1(A1), .S2(AX), .Y(Y2));

	mx51 MX51_3 (.A(X0[3]), .B(X1[3]), .C(X2[3]), 
						 .D(X3[3]), .E(XX[3]), .S0(A0), 
						 .S1(A1), .S2(AX), .Y(Y3));

	mx51 MX51_4 (.A(X0[4]), .B(X1[4]), .C(X2[4]), 
						 .D(X3[4]), .E(XX[4]), .S0(A0), 
						 .S1(A1), .S2(AX), .Y(Y4));

	mx51 MX51_5 (.A(X0[5]), .B(X1[5]), .C(X2[5]), 
						 .D(X3[5]), .E(XX[5]), .S0(A0), 
						 .S1(A1), .S2(AX), .Y(Y5));

	mx51 MX51_6 (.A(X0[6]), .B(X1[6]), .C(X2[6]), 
						 .D(X3[6]), .E(XX[6]), .S0(A0), 
						 .S1(A1), .S2(AX), .Y(Y6));

	mx51 MX51_7 (.A(X0[7]), .B(X1[7]), .C(X2[7]), 
						 .D(X3[7]), .E(XX[7]), .S0(A0), 
						 .S1(A1), .S2(AX), .Y(Y7));

	assign Y[0] = Y0;
	assign Y[1] = Y1;
	assign Y[2] = Y2;
	assign Y[3] = Y3;
	assign Y[4] = Y4;
	assign Y[5] = Y5;
	assign Y[6] = Y6;
	assign Y[7] = Y7;
	
endmodule

// MUX 7 TO 1
module mx71(
	  input A,
	  input B,
	  input C,
	  input D,
	  input E,
	  input F,
	  input G,
	  input S0,
	  input S1,
	  input S2,	  
	  output Y);

	wire SLA = (!S0 & !S1 & !S2);
	wire SLB = ( S0 & !S1 & !S2);
	wire SLC = (!S0 &  S1 & !S2);
	wire SLD = ( S0 &  S1 & !S2);
	wire SLE = (!S0 & !S1 &  S2);
	wire SLF = ( S0 & !S1 &  S2);
	wire SLG = (!S0 &  S1 &  S2);
	
	wire YA = A & SLA;
	wire YB = B & SLB;
	wire YC = C & SLC;
	wire YD = D & SLD;
	wire YE = E & SLE;
	wire YF = F & SLF;
	wire YG = G & SLG;
	
	assign Y = (YA | YB | YC | YD | YE | YF | YG);
endmodule

// BUS 8 BITS MUX 7 TO 1
module mx7(
    input [7:0] X0,
    input [7:0] X1,
    input [7:0] X2,
    input [7:0] X3,
    input [7:0] X4,
    input [7:0] X5,
    input [7:0] X6,
    input A0,
    input A1,
    input A2,
    output [7:0] Y
    );

	wire Y0;
	wire Y1;
	wire Y2;
	wire Y3;
	wire Y4;
	wire Y5;
	wire Y6;
	wire Y7;

	mx71 MX71_0 (.A(X0[0]), .B(X1[0]), .C(X2[0]), 
					 .D(X3[0]), .E(X4[0]), .F(X5[0]), 
					 .G(X6[0]), .S0(A0), .S1(A1), .S2(A2), .Y(Y0));

	mx71 MX71_1 (.A(X0[1]), .B(X1[1]), .C(X2[1]), 
					 .D(X3[1]), .E(X4[1]), .F(X5[1]), 
					 .G(X6[1]), .S0(A0), .S1(A1), .S2(A2), .Y(Y1));

	mx71 MX71_2 (.A(X0[2]), .B(X1[2]), .C(X2[2]), 
					 .D(X3[2]), .E(X4[2]), .F(X5[2]), 
					 .G(X6[2]), .S0(A0), .S1(A1), .S2(A2), .Y(Y2));

	mx71 MX71_3 (.A(X0[3]), .B(X1[3]), .C(X2[3]), 
					 .D(X3[3]), .E(X4[3]), .F(X5[3]), 
					 .G(X6[3]), .S0(A0), .S1(A1), .S2(A2), .Y(Y3));

	mx71 MX71_4 (.A(X0[4]), .B(X1[4]), .C(X2[4]), 
					 .D(X3[4]), .E(X4[4]), .F(X5[4]), 
					 .G(X6[4]), .S0(A0), .S1(A1), .S2(A2), .Y(Y4));

	mx71 MX71_5 (.A(X0[5]), .B(X1[5]), .C(X2[5]), 
					 .D(X3[5]), .E(X4[5]), .F(X5[5]), 
					 .G(X6[5]), .S0(A0), .S1(A1), .S2(A2), .Y(Y5));

	mx71 MX71_6 (.A(X0[6]), .B(X1[6]), .C(X2[6]), 
					 .D(X3[6]), .E(X4[6]), .F(X5[6]), 
					 .G(X6[6]), .S0(A0), .S1(A1), .S2(A2), .Y(Y6));

	mx71 MX71_7 (.A(X0[7]), .B(X1[7]), .C(X2[7]), 
					 .D(X3[7]), .E(X4[7]), .F(X5[7]), 
					 .G(X6[7]), .S0(A0), .S1(A1), .S2(A2), .Y(Y7));

	assign Y[0] = Y0;
	assign Y[1] = Y1;
	assign Y[2] = Y2;
	assign Y[3] = Y3;
	assign Y[4] = Y4;
	assign Y[5] = Y5;
	assign Y[6] = Y6;
	assign Y[7] = Y7;

endmodule

// 8 bit COMPARATOR
// Y = 0 when A = B
module comp8b(A, B, Y);

	input [7:0] A;
	input [7:0] B;
	output Y;

	wire Y0 = A[0] ^ B[0];
	wire Y1 = A[1] ^ B[1];
	wire Y2 = A[2] ^ B[2];
	wire Y3 = A[3] ^ B[3];
	wire Y4 = A[4] ^ B[4];
	wire Y5 = A[5] ^ B[5];
	wire Y6 = A[6] ^ B[6];
	wire Y7 = A[7] ^ B[7];
	
	assign Y = (Y0 | Y1 | Y2 | Y3 | Y4 | Y5 | Y6 | Y7);

endmodule

// 8 BITS DATA REGISTER
// register cleared at power up
module reg8(C, CE, CLR, D, Q);

	input C;
	input CE;
	input CLR;
	input [7:0] D;
	output [7:0] Q;
   
   
   FDCE #( .INIT(1'b0) ) Q0 (.C(C), 
            .CE(CE), 
            .CLR(CLR), 
            .D(D[0]), 
            .Q(Q[0]));
   FDCE #( .INIT(1'b0) ) Q1 (.C(C), 
            .CE(CE), 
            .CLR(CLR), 
            .D(D[1]), 
            .Q(Q[1]));
   FDCE #( .INIT(1'b0) ) Q2 (.C(C), 
            .CE(CE), 
            .CLR(CLR), 
            .D(D[2]), 
            .Q(Q[2]));
   FDCE #( .INIT(1'b0) ) Q3 (.C(C), 
            .CE(CE), 
            .CLR(CLR), 
            .D(D[3]), 
            .Q(Q[3]));
   FDCE #( .INIT(1'b0) ) Q4 (.C(C), 
            .CE(CE), 
            .CLR(CLR), 
            .D(D[4]), 
            .Q(Q[4]));
   FDCE #( .INIT(1'b0) ) Q5 (.C(C), 
            .CE(CE), 
            .CLR(CLR), 
            .D(D[5]), 
            .Q(Q[5]));
   FDCE #( .INIT(1'b0) ) Q6 (.C(C), 
            .CE(CE), 
            .CLR(CLR), 
            .D(D[6]), 
            .Q(Q[6]));
   FDCE #( .INIT(1'b0) ) Q7 (.C(C), 
            .CE(CE), 
            .CLR(CLR), 
            .D(D[7]), 
            .Q(Q[7]));
endmodule

// MCR/PCR DATA REGISTER
module mcrreg(PHI1, CLR, WR, A0, A1, A2, A3, DIN,
				  PC0, PC1, PC2, PC3, MC);

	input PHI1;
	input CLR;
	input WR;
	input A0;
	input A1;
	input A2;
	input A3;
	input [7:0] DIN;
	output [7:0] PC0;
	output [7:0] PC1;
	output [7:0] PC2;
	output [7:0] PC3;
	output [7:0] MC;

   wire [7:0] MCIN;
   wire [7:0] PC0_DUMMY;
   wire [7:0] PC1_DUMMY;
   wire [7:0] PC2_DUMMY;
   wire [7:0] PC3_DUMMY;

	wire WPC0 = (WR & !A3 & !A2 & !A1 & !A0); // write 00 -- preconfiguration reg. 0
	wire WPC1 = (WR & !A3 & !A2 & !A1 &  A0); // write 01 -- preconfiguration reg. 1
	wire WPC2 = (WR & !A3 & !A2 &  A1 & !A0); // write 02 -- preconfiguration reg. 2
	wire WPC3 = (WR & !A3 & !A2 &  A1 &  A0); // write 03 -- preconfiguration reg. 3
	wire WMC4 = (WR & !A3 &  A2 & !A1 & !A0); // write 04 -- memory configuration reg.
	wire WMC0 = (WR &  A3 & !A2 & !A1 & !A0); // write 08 (LCR 0)
	wire WMC1 = (WR &  A3 & !A2 & !A1 &  A0); // write 09 (LCR 1)
	wire WMC2 = (WR &  A3 & !A2 &  A1 & !A0); // write 0A (LCR 2)
	wire WMC3 = (WR &  A3 & !A2 &  A1 &  A0); // write 0B (LCR 3)
	wire WMC  = (WMC0 | WMC1 | WMC2 | WMC3 | WMC4);

	// write PCR 0
   reg8  PCR0 (.C(PHI1), .CE(WPC0), .CLR(CLR), .D(DIN[7:0]), .Q(PC0_DUMMY[7:0]));

	// write PCR 1
   reg8  PCR1 (.C(PHI1), .CE(WPC1), .CLR(CLR), .D(DIN[7:0]), .Q(PC1_DUMMY[7:0]));

	// write PCR 2
   reg8  PCR2 (.C(PHI1), .CE(WPC2), .CLR(CLR), .D(DIN[7:0]), .Q(PC2_DUMMY[7:0]));
	
	// write PCR 3
   reg8  PCR3 (.C(PHI1), .CE(WPC3), .CLR(CLR), .D(DIN[7:0]), .Q(PC3_DUMMY[7:0]));

	// MCIN (data in for MCR) is DIN (A3 = 0) or LCR (A3 = 1)
   mx5  MXMCR (.AX(A3), 
               .A0(A0), 
               .A1(A1), 
               .XX(DIN[7:0]), 
               .X0(PC0_DUMMY[7:0]), 
               .X1(PC1_DUMMY[7:0]), 
               .X2(PC2_DUMMY[7:0]), 
               .X3(PC3_DUMMY[7:0]), 
               .Y(MCIN[7:0]));

	// write MCR
   reg8  MCR (.C(PHI1), .CE(WMC), .CLR(CLR), .D(MCIN[7:0]), .Q(MC[7:0]));

   assign PC0[7:0] = PC0_DUMMY[7:0];
   assign PC1[7:0] = PC1_DUMMY[7:0];
   assign PC2[7:0] = PC2_DUMMY[7:0];
   assign PC3[7:0] = PC3_DUMMY[7:0];
	
endmodule

// ALL DATA REGISTERS
module allregs(PHI1, CLR, WR, A0, A1, A2, A3, DIN,
					DOUT, MC, PR, CR);

	input PHI1;
	input CLR;
	input WR;
	input A0;
	input A1;
	input A2;
	input A3;
	input [7:0] DIN;
   output [7:0] DOUT;
	output [7:0] MC;
	output [7:0] PR;
	output [7:0] CR;

   wire [7:0] CR_DUMMY;
   wire [7:0] PR_DUMMY;
   wire [7:0] PC0_DUMMY;
   wire [7:0] PC1_DUMMY;
   wire [7:0] PC2_DUMMY;
   wire [7:0] PC3_DUMMY;
   wire [7:0] MC_DUMMY;

	wire WPR = (WR & !A3 &  A2 & !A1 &  A0); // write 05 -- page 0 relocation
	wire WCR = (WR & !A3 &  A2 &  A1 &  A0); // write 06 -- control register

	// write PR
   reg8  PRR (.C(PHI1), .CE(WPR), .CLR(CLR), .D(DIN[7:0]), .Q(PR_DUMMY[7:0]));

	// write CR
   reg8  CRR (.C(PHI1), .CE(WCR), .CLR(CLR), .D(DIN[7:0]), .Q(CR_DUMMY[7:0]));

	// write PCR/MCR
	mcrreg MCR (.PHI1(PHI1), .CLR(CLR), .WR(WR), .A0(A0), .A1(A1), .A2(A2), .A3(A2),
				   .DIN(DIN[7:0]), .PC0(PC0_DUMMY[7:0]), .PC1(PC1_DUMMY[7:0]),
				   .PC2(PC2_DUMMY[7:0]), .PC3(PC3_DUMMY[7:0]), .MC(MC_DUMMY[7:0]));

	// data out
   mx7  MXDATA (.A0(A0), .A1(A1), .A2(A2), 
               .X0(PC0_DUMMY[7:0]), .X1(PC1_DUMMY[7:0]), .X2(PC2_DUMMY[7:0]), 
               .X3(PC3_DUMMY[7:0]), .X4(MC_DUMMY[7:0]), .X5(PR_DUMMY[7:0]), 
               .X6(CR_DUMMY[7:0]), .Y(DOUT[7:0]));

   assign CR[7:0] = CR_DUMMY[7:0];
   assign MC[7:0] = MC_DUMMY[7:0];
   assign PR[7:0] = PR_DUMMY[7:0];

endmodule

// bidirectional data buffer
module iobuffer8 (DOUT, E, DIN, D);

	input [7:0] DOUT;
	input E;
	output [7:0] DIN;
	inout [7:0] D;
   
   IOBUFE  IOB0 (.E(E), 
                .I(DOUT[0]), 
                .O(DIN[0]), 
                .IO(D[0]));
   IOBUFE  IOB1 (.E(E), 
                .I(DOUT[1]), 
                .O(DIN[1]), 
                .IO(D[1]));
   IOBUFE  IOB2 (.E(E), 
                .I(DOUT[2]), 
                .O(DIN[2]), 
                .IO(D[2]));
   IOBUFE  IOB3 (.E(E), 
                .I(DOUT[3]), 
                .O(DIN[3]), 
                .IO(D[3]));
   IOBUFE  IOB4 (.E(E), 
                .I(DOUT[4]), 
                .O(DIN[4]), 
                .IO(D[4]));
   IOBUFE  IOB5 (.E(E), 
                .I(DOUT[5]), 
                .O(DIN[5]), 
                .IO(D[5]));
   IOBUFE  IOB6 (.E(E), 
                .I(DOUT[6]), 
                .O(DIN[6]), 
                .IO(D[6]));
   IOBUFE  IOB7 (.E(E), 
                .I(DOUT[7]), 
                .O(DIN[7]), 
                .IO(D[7]));
endmodule

// DBE = data bus enable (active H)
module mmuregs(RES, PHI0, RW, DBE, PG01, YC, A, TA, MC, CR, D); 

	input RES;
	input PHI0;
	input RW;
	input DBE;	
	input [15:0] A;
	(* NOREDUCE = "YES" *)
	output PG01;
	(* NOREDUCE = "YES" *)	
	output YC;	
	output [15:8] TA;
	output [7:0] MC;
	output [7:0] CR;
	inout [7:0] D;
	
	wire CLR = !RES;
	wire PHI1 = !PHI0;
	wire [7:0] PR;
	wire [7:0] DIN;
	wire [7:0] DOUT;
	wire [7:0] AH;
	wire [7:0] TH;
	(* NOREDUCE = "YES" *)
	wire YCMP;
   wire A0 = A[0];
   wire A1 = A[1];
   wire A2 = A[2];
   wire A3 = A[3];
   wire A4 = A[4];
   wire A5 = A[5];
   wire A6 = A[6];
   wire A7 = A[7];
   wire A8 = A[8];
   wire A9 = A[9];
   wire A10 = A[10];
   wire A11 = A[11];
   wire A12 = A[12];
   wire A13 = A[13];
   wire A14 = A[14];
   wire A15 = A[15];

	assign AH[7:0] = A[15:8];

	assign TA[15:8] = TH[7:0];
	
	// PAGE 0 SELECT
	wire PG0 = RES & !A15 & !A14 & !A13 & !A12 & !A11 & !A10 & !A9 & !A8;
	// PAGE 1 SELECT
	wire PG1 = RES & !A15 & !A14 & !A13 & !A12 & !A11 & !A10 & !A9 &  A8;
	
	// MMU SELCT: FE00 - FE0F
	wire MMUSEL = RES & A15 & A14 & A13 & A12 & A11 & A10 & A9 & !A8 & !A7 & !A6 & !A5 & !A4;
	// write to MMU reg.
	wire WR = MMUSEL & !RW;
	// read MMU reg
	wire OE = MMUSEL & RW & DBE;

	// data buffer
   iobuffer8 IOBUF8 (.DOUT(DOUT[7:0]), .E(OE), 
                     .DIN(DIN[7:0]), .D(D[7:0]));
	
	// write to MMU reg.
	allregs MMUR (.PHI1(PHI1), .CLR(CLR), .WR(WR), .A0(A0), .A1(A1), .A2(A2), .A3(A3), 
                 .DIN(DIN[7:0]), .DOUT(DOUT[7:0]), .MC(MC[7:0]), .PR(PR[7:0]),
                 .CR(CR[7:0]));
						 
   comp8b ACMP (.A(PR[7:0]), .B(AH[7:0]), .Y(YCMP));
   //comp8b ACMP (.A(PR[7:0]), .B(A[15:8]), .Y(YCMP));

	// translated address
   mx2e TMUX (.E(YCMP), .S0(PG0), .X0(AH[7:0]), .X1(PR[7:0]), .Y(TH[7:0]));
   //mx2e TMUX (.E(YCMP), .S0(PG0), .X0(A[15:8]), .X1(PR[7:0]), .Y(TA[15:8]));

	assign PG01 = !(PG0 | PG1);
	//assign BNK0 = (YCMP & PG01);
	assign YC = YCMP;

endmodule

/*
	MAIN MODULE
	
	IN
		/RES			reset, active low
		PHI0			CPU PHI0 input
		RW				CPU R/W signal
		DBE			data bus enable (CPU read registers)
		A15..A0		CPU address lines
		
	IN/OUT
		D7..D0		CPU data bus
		
	OUT
		TA16..TA8	translated RAM address lines
		MC5..MC0		reflect bit 5..bit 0 of MCR (memory configuration reg.)
		CR7..CR3		reflect bit 7..bit 3 of CR (control register)
		CS0			select first 128k ram bank (active high)
		CS1			select 2'nd 128k ram bank (active high)	
*/

module clone_8722(RES, PHI0, RW, DBE, A, D, TA, MC0, MC1, MC2, MC3, MC4, MC5, CS0, CS1, CR3, CR4, CR5, CR6, CR7);

	input RES;
	input PHI0;
	input RW;
	input DBE;
	input [15:0] A;
   inout [7:0] D;
	(* NOREDUCE = "YES,YES,YES,YES,YES,YES,YES,YES,YES" *) 
	output [16:8] TA; 	
	output MC0;
	output MC1;
	output MC2;
	output MC3;
	output MC4;
	output MC5;
	(* NOREDUCE = "YES" *)
	output CS0;
	(* NOREDUCE = "YES" *)	
	output CS1;
	output CR3;
	output CR4;
	output CR5;
	output CR6;
	output CR7;
	
	wire [7:0] MC;
	wire [7:0] CR;
	wire [15:8] TA_DUMMY;
	(* NOREDUCE = "YES" *)	
	wire PG01;
	(* NOREDUCE = "YES" *)
	wire YC;

	mmuregs MMUR(.RES(RES), .PHI0(PHI0), .RW(RW), .DBE(DBE), .PG01(PG01), .YC(YC),
					 .A(A[15:0]), .TA(TA_DUMMY[15:8]), .MC(MC[7:0]), .CR(CR[7:0]), .D(D[7:0])); 
					 
	assign CR3 = CR[3];
	assign CR4 = CR[4];
	assign CR5 = CR[5];
	assign CR6 = CR[6];
	assign CR7 = CR[7];
	
	assign MC0 = MC[0];
	assign MC1 = MC[1];
	assign MC2 = MC[2];
	assign MC3 = MC[3];
	assign MC4 = MC[4];
	assign MC5 = MC[5];
	wire MC6 = MC[6];
	wire MC7 = MC[7];

	wire CR0 = CR[0];
	wire CR1 = CR[1];
	wire CR2 = CR[2];

	assign TA[8] = TA_DUMMY[8];
	assign TA[9] = TA_DUMMY[9];
	assign TA[10] = TA_DUMMY[10];
	assign TA[11] = TA_DUMMY[11];
	assign TA[12] = TA_DUMMY[12];
	assign TA[13] = TA_DUMMY[13];
	assign TA[14] = TA_DUMMY[14];
	assign TA[15] = TA_DUMMY[15];

	wire B1K = !A[15] & !A[14] & !A[13] & !A[12] & !A[11] & !A[10];
	wire B2K = !A[15] & !A[14] & !A[13] & !A[12] & !A[11];
	wire B4K = !A[15] & !A[14] & !A[13] & !A[12];
	wire B8K = !A[15] & !A[14] & !A[13];
	
	wire CMR1 = (B1K & CR2 & !CR0 & !CR1);
	wire CMR2 = (B2K & CR2 & !CR0 &  CR1);
	wire CMR3 = (B4K & CR2 &  CR0 & !CR1);
	wire CMR4 = (B8K & CR2 &  CR0 &  CR1);
	(* NOREDUCE = "YES" *)
	wire CMRB = !(CMR1 | CMR2 | CMR3 | CMR4);
	
	assign TA[16] = MC6 & CMRB & PG01 & YC;
	assign CS1 = MC7 & CMRB & PG01 & YC;
	assign CS0 = !CS1;

endmodule
